Package: BeadDataPackR
Type: Package
Title: Compression of Illumina BeadArray data
Version: 1.65.0
Date: 2018-02-09
Author: Mike Smith, Andy Lynch
Maintainer: Mike Smith <grimbough@gmail.com>
Description: Provides functionality for the compression and decompression of raw bead-level data from the Illumina BeadArray platform.
License: GPL-2
LazyLoad: yes
biocViews: Microarray
VignetteBuilder: knitr
Suggests: BiocStyle, knitr
Imports: stats, utils
git_url: https://git.bioconductor.org/packages/BeadDataPackR
git_branch: devel
git_last_commit: 1ecd28a
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 21:25:42 UTC; biocbuild
