\name{example}
\alias{example}
\docType{data}
\title{Example bead-level data}
\description{
Example bead-level data consisting of a \code{.txt} file, a \code{.locs} file and the \code{.bab} file that is produced from their compression.
}

\keyword{datasets}
